/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web;

import org.apache.struts.util.ModuleException;

/**
 * Thrown by <code>Action</code> methods to indicate the requested operation
 * has failed.
 * 
 * @author rlogiacco@smartlab.net
 */
public class ActionException extends ModuleException {

	private static final long serialVersionUID = 5024885398839776328L;

	/**
	 * Instantiates the class with a message. The providen message is used as a
	 * key and searched in available resource bundles. If the message is not a
	 * valid resource key than it's used directly.
	 * 
	 * @param message the key to be searched in resource bundles.
	 */
	public ActionException(String message) {
		super(message);
	}

	/**
	 * Instantiates the class with a message and a parameter. The providen
	 * message is used as a key and searched in available resource bundles. If
	 * the message is not a valid resource key than it's used directly.
	 * 
	 * @param message the key to be searched in resource bundles.
	 * @param param an object to be used for param substitution.
	 */
	public ActionException(String message, Object param) {
		super(message, param);
	}

	/**
	 * Instantiates the class with a message and two parameters. The providen
	 * message is used as a key and searched in available resource bundles. If
	 * the message is not a valid resource key than it's used directly.
	 * 
	 * @param message the key to be searched in resource bundles.
	 * @param param1 an object to be used for param substitution.
	 * @param param2 an object to be used for param substitution.
	 */
	public ActionException(String message, Object param1, Object param2) {
		super(message, param1, param2);
	}

	/**
	 * Instantiates the class with a message and three parameters. The providen
	 * message is used as a key and searched in available resource bundles. If
	 * the message is not a valid resource key than it's used directly.
	 * 
	 * @param message the key to be searched in resource bundles.
	 * @param param1 an object to be used for param substitution.
	 * @param param2 an object to be used for param substitution.
	 * @param param3 an object to be used for param substitution.
	 */
	public ActionException(String message, Object param1, Object param2, Object param3) {
		super(message, param1, param2, param3);
	}

	/**
	 * Instantiates the class with a message and four parameters. The providen
	 * message is used as a key and searched in available resource bundles. If
	 * the message is not a valid resource key than it's used directly.
	 * 
	 * @param message the key to be searched in resource bundles.
	 * @param param1 an object to be used for param substitution.
	 * @param param2 an object to be used for param substitution.
	 * @param param3 an object to be used for param substitution.
	 * @param param4 an object to be used for param substitution.
	 */
	public ActionException(String message, Object param1, Object param2, Object param3, Object param4) {
		super(message, param1, param2, param3, param4);
	}

	/**
	 * Instantiates the class with a message and an array of parameters. The
	 * providen message is used as a key and searched in available resource
	 * bundles. If the message is not a valid resource key than it's used
	 * directly.
	 * 
	 * @param message the key to be searched in resource bundles.
	 * @param params a set of objects for param substitution.
	 */
	public ActionException(String message, Object[] params) {
		super(message, params);
	}
}