/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.StaleObjectStateException;
import org.hibernate.Transaction;

/**
 * @author rlogiacco@smartlab.net
 */
public abstract class Domain {

	/* # net.smartlab.web.BusinessException de */
	/* # net.smartlab.web.DataAccessObject dao */
	/* # net.smartlab.web.DataTransferObject dto */

	/**
	 * @TODO documentation Comment for <code>logger</code>
	 */
	protected final Log logger = LogFactory.getLog(this.getClass());

	/**
	 * @TODO documentation
	 * @param factory
	 * @return
	 * @throws BusinessException
	 */
	protected Transaction begin(BusinessObjectFactory factory) throws BusinessException {
		try {
			return factory.current().beginTransaction();
		} catch (HibernateException he) {
			logger.error("[ smartweb ] failed to begin transaction");
			throw new BusinessException("persistance.error.begin", he);
		} catch (DAOException boe) {
			throw new BusinessException(boe);
		}
	}

	/**
	 * @TODO documentation
	 * @param transaction
	 * @throws BusinessException
	 */
	protected void commit(Transaction transaction) throws BusinessException {
		try {
			transaction.commit();
		} catch (StaleObjectStateException sose) {
			logger.info("[ smartweb ] optimistical locking collision");
			throw new BusinessException("persistance.locking.collision", sose);
		} catch (HibernateException he) {
			logger.error("[ smartweb ] failed to commit transaction");
			throw new BusinessException("persistance.error.commit", he);
		}
	}

	/**
	 * @TODO documentation
	 * @param transaction
	 * @throws BusinessException
	 */
	protected void rollback(Transaction transaction) throws BusinessException {
		try {
			transaction.rollback();
		} catch (HibernateException he) {
			logger.warn("[ smartweb ] failed to rollback transaction");
		}
	}

	/**
	 * Subclasses should make their own constructors private and behave like
	 * singletons.
	 */
	protected Domain() {
		super();
	}
	
	/**
	 * @TODO documentation 
	 * @param domain
	 * @return
	 */
	public static Domain getInstance(Domain domain) {
		ClassLoader loader = domain.getClass().getClassLoader();
		Class[] interfaces = domain.getClass().getInterfaces();
		return (Domain)java.lang.reflect.Proxy.newProxyInstance(loader, interfaces, new Wrapper(domain));
	}

	/**
	 * @TODO documentation 
	 * @author rlogiacco@smartlab.net
	 */
	private static class Wrapper implements InvocationHandler {

		/**
		 * The wrapped implementation.
		 */
		private Domain domain;

		private Wrapper(Domain domain) {
			this.domain = domain;
		}
		
		/**
		 * @see java.lang.reflect.InvocationHandler#invoke(java.lang.Object, java.lang.reflect.Method, java.lang.Object[])
		 */
		public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
			System.out.print("(");
			Object result = null;
			try {
				result = method.invoke(domain, args);
			} catch (InvocationTargetException ite) {
				throw ite.getCause();
			}
			System.out.print(")");
			return result;
		}
	}
}