/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web;

import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import org.hibernate.Hibernate;
import org.hibernate.HibernateException;
import org.hibernate.usertype.UserType;

/**
 * @TODO documentation
 * 
 * @author rlogiacco@smartlab.net
 */
public abstract class Enumeration implements UserType, Serializable {

    private final static long serialVersionUID = 8849967605617496075L;

    /**
     * @TODO documentation Comment for <code>TYPES</code>
     */
    private final static int[] TYPES = new int[] {
        Types.SMALLINT
    };

    /**
     * @TODO documentation Comment for <code>id</code>
     */
    private final int id;

    /**
     * @TODO documentation Comment for <code>display</code>
     */
    private final String display;


    /**
     * @TODO documentation
     */
    protected Enumeration() {
        // Default constructor
        this.id = -1;
        this.display = null;
    }

    /**
     * @TODO documentation
     * 
     * @param id
     * @param display
     */
    protected Enumeration(int id, String display) {
        this.id = id;
        this.display = display;
    }

    /**
     * @TODO documentation
     * 
     * @return
     */
    public int getId() {
        return id;
    }

    /**
     * @TODO documentation
     * 
     * @return
     */
    public String getDisplay() {
        return display;
    }

    /**
     * Decodes a unique identifier into an instance of this class. This method
     * <b>can not</b> return <code>null</code> values, instead it should
     * define a <i>default</i> value.
     * 
     * @param id the identifier to decode.
     * @return an instance of this class.
     */
    public abstract Enumeration decode(int id);

    /**
     * @see org.hibernate.usertype.UserType#deepCopy(java.lang.Object)
     */
    public Object deepCopy(Object value) throws HibernateException {
        return value;
    }

    /**
     * @see org.hibernate.usertype.UserType#equals(java.lang.Object,
     *      java.lang.Object)
     */
    public boolean equals(Object src, Object dst) throws HibernateException {
        if (src == dst) {
            return true;
        }
        if (src == null || dst == null) {
            return false;
        }
        return Hibernate.INTEGER.isEqual(src, dst);

    }

    /**
     * @see org.hibernate.usertype.UserType#isMutable()
     */
    public boolean isMutable() {
        return false;
    }

    /**
     * @see org.hibernate.usertype.UserType#nullSafeGet(java.sql.ResultSet,
     *      java.lang.String[], java.lang.Object)
     */
    public Object nullSafeGet(ResultSet rows, String[] names, Object value) throws HibernateException, SQLException {
        int id = ((Integer)Hibernate.INTEGER.nullSafeGet(rows, names[0])).intValue();
        return this.decode(id);

    }

    /**
     * @see org.hibernate.usertype.UserType#nullSafeSet(java.sql.PreparedStatement,
     *      java.lang.Object, int)
     */
    public void nullSafeSet(PreparedStatement statement, Object value, int index) throws HibernateException,
            SQLException {
        try {
            statement.setInt(index, ((Enumeration)value).getId());
        } catch (ClassCastException cce) {
            throw new HibernateException(cce);
        }
    }

    /**
     * @see org.hibernate.usertype.UserType#returnedClass()
     */
    public Class returnedClass() {
        return this.getClass();
    }

    /**
     * @see org.hibernate.usertype.UserType#sqlTypes()
     */
    public int[] sqlTypes() {
        return TYPES;
    }

    /**
     * @see org.hibernate.usertype.UserType#assemble(java.io.Serializable,
     *      java.lang.Object) TODO implement
     */
    public Object assemble(Serializable arg0, Object arg1) throws HibernateException {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * @see org.hibernate.usertype.UserType#disassemble(java.lang.Object) TODO
     *      implement
     */
    public Serializable disassemble(Object arg0) throws HibernateException {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * @see org.hibernate.usertype.UserType#hashCode(java.lang.Object) TODO
     *      implement
     */
    public int hashCode(Object obj) throws HibernateException {
        return ((Enumeration)obj).getId();
    }

    /**
     * @see org.hibernate.usertype.UserType#replace(java.lang.Object,
     *      java.lang.Object, java.lang.Object) TODO implement
     */
    public Object replace(Object arg0, Object arg1, Object arg2) throws HibernateException {
        // TODO Auto-generated method stub
        return null;
    }
}