/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web;

/**
 * A LockException is thrown whenever an application tries to update a
 * persistent representation into the storage system with an obsolete version of
 * datas or while another application has previously locked the datas, depending
 * on the locking mechanism adopted.
 * 
 * @author rlogiacco@smartlab.net
 */
public class LockException extends DAOException {

	private final static long serialVersionUID = 6434249797441671104L;

	/**
	 * Default empty constructor.
	 */
	public LockException() {
		super();
	}

	/**
	 * Constructs a new instance with the specified describing message.
	 * 
	 * @param message the description of the occurred exception.
	 */
	public LockException(String message) {
		super(message);
	}

	/**
	 * Constructs a new instance with the specified cause.
	 * 
	 * @param cause the <code>throwable</code> instance wich generated this
	 *            exception.
	 */
	public LockException(Throwable cause) {
		super(cause);
	}

	/**
	 * Constructs a new instance with the specified describing message and
	 * cause.
	 * 
	 * @param message the description of the occurred exception.
	 * @param cause the <code>throwable</code> instance wich generated this
	 *            exception.
	 */
	public LockException(String message, Throwable cause) {
		super(message, cause);
	}
}