/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web.config;

import java.net.URL;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import net.smartlab.config.Element;
import net.smartlab.config.XMLConfiguration;
import net.smartlab.web.BusinessObjectFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

/**
 * The Hibernate configuration file used must reside into the
 * <code>META-INF</code> directory of the topmost packaging archive and should
 * be named <code>smartweb.jar.hcf</code> or have the name of the JAR file
 * containing the subclass followed by the <code>.hcf</code> suffix.
 * 
 * @author rlogiacco@smartlab.net
 */
public class JNDIConfigurationStrategy implements ConfigurationStrategy {

	/**
	 * Provides logging capabilities to the strategy.
	 */
	protected final Log logger = LogFactory.getLog(JNDIConfigurationStrategy.class);

	/**
	 * @see net.smartlab.web.config.ConfigurationStrategy#getSessionFactory(net.smartlab.web.BusinessObjectFactory)
	 */
	public SessionFactory getSessionFactory(BusinessObjectFactory factory) {
		synchronized (BusinessObjectFactory.class) {
			String path = factory.getClass().getProtectionDomain().getCodeSource().getLocation().getFile();
			logger.debug("   trying to configure with `/META-INF" + path.substring(path.lastIndexOf('/')) + ".hcf`");
            URL file = factory.getClass().getResource("/META-INF" + path.substring(path.lastIndexOf('/')) + ".hcf");
			if (file == null) {
                logger.debug("   trying to configure with `/META-INF/smartweb.jar.hcf`");
                file = factory.getClass().getResource("/META-INF/smartweb.jar.hcf");
			}
			try {
				XMLConfiguration config = new XMLConfiguration(file);
				Element sessionFactory = config.getElement("session-factory");
				String jndi = sessionFactory.getAttribute("name");
				try {
					return (SessionFactory)PortableRemoteObject.narrow(new InitialContext().lookup(jndi), SessionFactory.class);
				} catch (NamingException ne) {
					logger.info("getSessionFactory() - configure hibernate");
					try {
						// Session factory not yet configured
						Configuration hibernate = new Configuration().configure(file);
                        hibernate.buildSessionFactory();
						return (SessionFactory)PortableRemoteObject.narrow(new InitialContext().lookup(jndi), SessionFactory.class);
					} catch (Exception e) {
						logger.error("getSessionFactory() - error", e);
					}
				}
			} catch (Exception e) {
				logger.error("getSessionFactory() - error", e);
			}
			return null;
		}
	}

}